@@ -1,13 +1,15 @@
-#' @keywords Internal
+#' @keywords internal
 check_index <- function(index, metadata) {
   ## check we have the correct number of dimensions
   if (isFALSE(length(index) == length(metadata$shape))) {
-    stop("The number of dimensions provided to 'index' does not match the shape of the array")
+    stop(
+      "The number of dimensions provided to 'index' does not match the shape of the array"
+    )
   }
 
   ## If any dimensions are NULL transform into the entirety of that dimension
   ## Otherwise check provided indices are valid
-  failed <- rep(FALSE, n = length(index))
+  failed <- rep_len(FALSE, length(index))
   for (i in seq_along(index)) {
     if (is.null(index[[i]])) {
       index[[i]] <- seq_len(metadata$shape[[i]])
@@ -37,34 +39,37 @@ check_index <- function(index, metadata) {
 #' @returns A character vector of length one containing the replacement
 #'   commands. This is expected to be passed to `parse() |> eval()`.
 #'
-#' @keywords Internal
+#' @keywords internal
 .create_replace_call <- function(x_name, idx_name, idx_length, y_name) {
-
   args <- sprintf("%s[[%d]]", idx_name, seq_len(idx_length))
-  args <- paste0(args, collapse = ",")
+  args <- paste(args, collapse = ",")
   cmd <- sprintf("%s[%s] <- %s", x_name, args, y_name)
-  
+
   return(cmd)
 }
 
 #' Parse the data type encoding string
-#' 
+#'
 #' @param typestr The datatype encoding string.  This is in the Numpy array
 #' typestr format.
-#' 
+#'
 #' @returns A list of length 4 containing the details of the data type.
+#'
+#' @keywords internal
 .parse_datatype <- function(typestr) {
   datatype <- list()
-  datatype_parts <- strsplit(typestr, "")[[1]]
+  datatype_parts <- strsplit(typestr, "", fixed = TRUE)[[1]]
 
-  datatype$endian <- switch(datatype_parts[1],
+  datatype$endian <- switch(
+    datatype_parts[1],
     "<" = "little",
     ">" = "big",
     "|" = NA
   )
 
-  datatype$base_type <- switch(datatype_parts[2],
-    "b" = "boolean",
+  datatype$base_type <- switch(
+    datatype_parts[2],
+    "b" = "bool",
     "i" = "int",
     "u" = "uint",
     "f" = "float",
@@ -77,15 +82,15 @@ check_index <- function(index, metadata) {
     "O" = "py_object"
   )
 
-  #datatype$nbytes <- as.integer(datatype_parts[3])
   datatype$nbytes <- as.integer(
     gsub(x = typestr, pattern = "^[<>|][[:alpha:]]", replacement = "")
   )
-  
-  if(datatype$base_type == 'unicode')
-    datatype$nbytes <- datatype$nbytes * 4
 
-  datatype$is_signed <- ifelse(datatype$base_type != "uint", TRUE, FALSE)
+  if (datatype$base_type == "unicode") {
+    datatype$nbytes <- datatype$nbytes * 4
+  }
+
+  datatype$is_signed <- datatype$base_type != "uint"
 
   return(datatype)
 }
@@ -99,19 +104,25 @@ check_index <- function(index, metadata) {
 #'
 #' @returns A character vector of length 1 containing the normalised path.
 #'
-#' @keywords Internal
+#' @keywords internal
 .normalize_array_path <- function(path) {
   ## we strip the protocol because it gets messed up by the slash removal later
   if (grepl(x = path, pattern = "^((https?://)|(s3://)).*$")) {
-    root <- gsub(x = path, pattern = "^((https?://)|(s3://)).*$", 
-                 replacement = "\\1")
-    path <- gsub(x = path, pattern = "^((https?://)|(s3://))(.*$)", 
-                 replacement = "\\4")
+    root <- gsub(
+      x = path,
+      pattern = "^((https?://)|(s3://)).*$",
+      replacement = "\\1"
+    )
+    path <- gsub(
+      x = path,
+      pattern = "^((https?://)|(s3://))(.*$)",
+      replacement = "\\4"
+    )
   } else {
     ## Replace all backward slash ("\\") with forward slash ("/")
     path <- gsub(x = path, pattern = "\\", replacement = "/", fixed = TRUE)
-    path <- normalizePath(path, winslash = "/", mustWork = FALSE)
-    root <- gsub(x = path, "(^[[:alnum:]:.]*/)(.*)", replacement = "\\1")
+    path <- R.utils::getAbsolutePath(path, expandTilde = TRUE)
+    root <- gsub(x = path, "(^[[:alnum:]:.]*/)?(.*)", replacement = "\\1")
     path <- gsub(x = path, "(^[[:alnum:]:.]*/)(.*)", replacement = "\\2")
   }
 
@@ -119,9 +130,9 @@ check_index <- function(index, metadata) {
   path <- gsub(x = path, pattern = "^/", replacement = "", fixed = FALSE)
   ## Strip any trailing "/" characters
   path <- gsub(x = path, pattern = "/$", replacement = "", fixed = FALSE)
-  ## Collapse any sequence of more than one "/" character into a single "/" 
+  ## Collapse any sequence of more than one "/" character into a single "/"
   path <- gsub(x = path, pattern = "//*", replacement = "/", fixed = FALSE)
-  ## The key prefix is then obtained by appending a single "/" character to 
+  ## The key prefix is then obtained by appending a single "/" character to
   ## the normalized logical path.
   path <- paste0(root, path, "/")
 
@@ -129,21 +140,40 @@ check_index <- function(index, metadata) {
 }
 
 .readVlenUTF8 <- function(input) {
-  
-  nvalues <- readBin(input, what = "integer", n = 1, size = 4)
+  con <- rawConnection(input)
+  on.exit(close(con))
+  nvalues <- readBin(con, what = "integer", n = 1, size = 4)
   output <- character(length = nvalues)
-  input <- tail(input, -4)
-  for(i in seq_len(nvalues)) {
-    
-    nbytes <- readBin(input, what = "integer", n = 1, size = 4)
-    input <- tail(input, -4)
-    if(nbytes > 0) {
-      output[i] <- rawToChar(input[seq_len(nbytes)])
-      input <- tail(input, -nbytes)
-    } 
+  for (i in seq_len(nvalues)) {
+    nbytes <- readBin(con, what = "integer", n = 1, size = 4)
+    output[i] <- readChar(con, nchars = nbytes, useBytes = TRUE)
   }
-  
+
   Encoding(output) <- "UTF-8"
   return(output)
-  
+}
+
+.file_or_blob_exists <- function(
+  zarr_array_path,
+  s3_client,
+  files
+) {
+  if (is.null(s3_client)) {
+    is_present <- setNames(
+      file.exists(paste0(zarr_array_path, files)),
+      files
+    )
+  } else {
+    parse_url <- parse_s3_path(zarr_array_path)
+    is_present <- vapply(
+      files,
+      FUN = function(f) {
+        key <- paste0(parse_url$object, f)
+        .s3_object_exists(s3_client, parse_url$bucket, key)
+      },
+      FUN.VALUE = logical(1)
+    )
+  }
+
+  return(is_present)
 }
